import assert from 'node:assert';
import serializerRaw, { wrap } from 'jest-snapshot-serializer-raw';
import { defaultGetRequestCacheKey } from './options.js';
import { fromResponseRecordText, toResponseRecordText } from './records.js';
const INTERNAL_HEADER = 'X-Jest-Playback-Internal';
export class PlaybackManager {
    constructor(runner, options = {}) {
        this.runner = runner;
        this.options = options;
        this.runner.expect.addSnapshotSerializer(serializerRaw);
    }
    async onRequest(request) {
        if (this.unwrapInternalRequest(request)) {
            return null; // avoid circular interception
        }
        const { currentTestName, snapshotState: { _counters, _snapshotData, _updateSnapshot }, } = this.runner.expect.getState();
        if (!currentTestName) {
            return null; // outside of test
        }
        const snapshotName = await this.getSnapshotName(request);
        const snapshotFullName = this.runner //
            .getSnapshotFullName(currentTestName, snapshotName);
        const snapshotIndex = _counters.get(snapshotFullName) ?? 0;
        const snapshotId = `${snapshotFullName} ${snapshotIndex + 1}`;
        const snapshot = _snapshotData[snapshotId]?.replace(/^\n|\n$/g, '');
        switch (_updateSnapshot) {
            case 'none':
                assert(snapshot, 'playback not found');
                return await this.play(snapshot, snapshotName);
            case 'new':
                return snapshot
                    ? await this.play(snapshot, snapshotName)
                    : await this.record(request, snapshotName);
            case 'all':
                return await this.record(request, snapshotName);
            default:
                _updateSnapshot;
                throw new Error(`Unexpected _updateSnapshot ${_updateSnapshot}`);
        }
    }
    async play(snapshot, name) {
        const response = fromResponseRecordText(snapshot);
        return await this.snapshot(response, name);
    }
    async record(request, name) {
        this.wrapInternalRequest(request);
        const response = await fetch(request);
        return await this.snapshot(response, name);
    }
    async snapshot(response, name) {
        this.runner
            .expect(wrap(await toResponseRecordText(response)))
            .toMatchSnapshot(name);
        return response;
    }
    wrapInternalRequest(request) {
        request.headers.set(INTERNAL_HEADER, '');
    }
    unwrapInternalRequest(request) {
        if (!request.headers.has(INTERNAL_HEADER)) {
            return false;
        }
        request.headers.delete(INTERNAL_HEADER);
        return true;
    }
    async getSnapshotName(request) {
        const { getRequestCacheKey = defaultGetRequestCacheKey } = this.options;
        return `(jest-playback) ${await getRequestCacheKey(request)}`;
    }
}
